/* 
################################################################################
#                                                                              #
# AUTHOR           : Andrew Fleming                                            #
#                                                                              #
# CREATION DATE    : 11 Feb 2008                                               #
#                                                                              #
# SHORT DESCRIPTION: AES Encryption/decryption standalone Tool                 #
#                                                                              #

################################################################################
*/
//#include <string>

//#include "CF_Utils.hpp"
//#include "GCrypt.hpp"
//#include "Logging.hpp"
//#include "VersionInfo.hpp"
#include <sys/types.h>
#include <sys/stat.h>
#include <windows.h>
#include "keyvalue.h"
#include "aescommon.h"
#include "aescrypt.hpp"

char AesUtil_cpp[]= "aesmain";

#define DEFAULT_BLOCKSIZE 8192

#define MODNAME AesUtil_cpp

void setupEnviroment(int argc, char **argv);
void showHelp();
int checkArgs();
bool checkFileExists(const char *filename);

bool encryptfile( const char *inputfile, const char *outputfile, const char *key, const char *override, const char *ciphertype,  const char *padlogfile);
bool decryptfile( const char *inputfile, const char *outputfile, const char *key, const char *override, const char *ciphertype,  const char *padlogfile);

int readblock(int rblocksize, int ifilehandle,char *rblock );
int writeblock(int wblocksize, int ofilehandle,char *wblock );
bool  processFile(   const char *optmode, const char *inputfile, const char *outputfile, const char *key, const char *override, const char *ciphertype, const char *padlogfile);
void displayPadding(FILE *fpadlogfile, int padBlock, int padLength, int byteread, const char *wblock);
struct keyvalue *kvlist[10];
int logging=1;
int main(int argc, char* argv[])
{

	
	bool processed;
   if ( argc < 5 ) 
   {
      showHelp();
      return -1;
   }

   setupEnviroment(argc, argv);

   if (checkArgs()!= 0){
		 MLOG(INFO_LOGLVL) "Missing options ");
		 return 0;
   }
   processed =processFile(  readArgs( "/OPTMODE=" ),
                 readArgs( "/IN=" ),
                 readArgs( "/OUT=" ),
				 readArgs( "/KEY=" ),
                 readArgs( "/OVERWRITE=" ),
                 readArgs( "/ENCRYPTION="),
                 readArgs( "/PADLOG="));

   MLOG(INFO_LOGLVL) " Operation %s %s ", readArgs( "/OPTMODE=" ), (processed==true)?"Succeeded":"Failed");
				 

   return 0;
}
void showHelp()
{
   fprintf( stdout, "usage: aesutil /optmode=<decrypt|encrypt> /in=<filename> /out=<filename> /key=<text string> </help> </fileoverwrite> </encryption=aes|des> </loglevel:X </padlog=<filename> >\n" );
   fprintf( stdout, "\t/optmode=<mode>    : Encryption or Decryption mode of operation \n" );
   fprintf( stdout, "\t/in=<filename>     : File path of input file\n" );
   fprintf( stdout, "\t/out=<filename>    : File path of the output file\n" );
   fprintf( stdout, "\t/fileoverwrite     : If the /out file exists overwrite it\n" );
   fprintf( stdout, "\t/encryption=<cipher> : Cipher to be used aes or des\n" );
   fprintf( stdout, "\t/key=<text string> : Key to be used for encryption/decryption operation must be 16(aes)/8(des) characters long\n\n" );
   fprintf( stdout, "\t/loglevel:<loglevel> logging level, valid values are 0-4, 0 meaning none, 4 meaning debug level\n\n" );
   fprintf( stdout, "\t/padlog=<filename> : File path of the output padding log file.\n" );

   fprintf( stdout, "\t/help : show the program usage\n" );

}

void setupEnviroment(int argc, char **argv)
{

	fprintf(stderr, "Entering setupEnvironment\n");
    loadArgs( argv, argc );

	fprintf(stderr, "Exiting setupEnvironment\n");

}

int checkArgs()
{
	fprintf(stderr, "Entering checkArgs\n");
    int errorCode=0;

    char *cipher   =readArgs ("/ENCRYPTION=");
    if(cipher==NULL || strlen(cipher)==0)
    {
        MLOG(ERROR_LOGLVL)" no /encryption used\n");
        errorCode|= EC_NO_CIPHER_SET;
    }
    char *optmode = readArgs( "/OPTMODE=" );
    if ( optmode ==NULL || strlen(optmode) == 0 ) 
    {
        MLOG( ERROR_LOGLVL) "no  /optmode used\n");
        errorCode|=EC_MODE_MISSING;
    }else if (stricmp(optmode, "decrypt") !=0 && 
		      stricmp(optmode, "encrypt") !=0)
			{
				errorCode |= EC_UNKNOWN_MODE;
			}

    MLOG(INFO_LOGLVL) "optmode selected: <%s>\n", optmode );

    char * inifile = readArgs( "/IN=" );
    if ( inifile == NULL ||strlen(inifile) == 0 ) 
    {
        MLOG(ERROR_LOGLVL ) "empty input filename found\n");
        errorCode|=EC_INPUTFILE_NAME_MISSING;
    }
	if (!checkFileExists(inifile)){
		 MLOG(ERROR_LOGLVL ) "input filename cannot be found\n");
        // input file exists.
		errorCode|=EC_INPUTFILE_DOESNOTEXIST;

    }


    MLOG(INFO_LOGLVL) "inifile selected: <%s>\n", inifile );

    char * outfile = readArgs("/OUT=" );
    if ( outfile == NULL || strlen(outfile) == 0 ) 
    {
        MLOG( ERROR_LOGLVL ) "empty output filename found\n");
        errorCode|=EC_OUTPUTFILE_NAME_MISSING;
        //TODO Add error to list of errors
    }else if (checkFileExists(outfile)){
              // Output file exists, check whether it should be overwritten
			  char * fileoverwrite=readArgs( "/FILEOVERWRITE=" );
              if(fileoverwrite ==NULL || stricmp(fileoverwrite,"y")!=0){
				  MLOG( ERROR_LOGLVL ) " output filename already exists, no overwrite selected\n");
				  errorCode|=EC_OUTPUTFILE_ALREADY_EXISTS;
                     
              }
    }

    MLOG(INFO_LOGLVL) "outfile selected: <%s>\n", outfile );

	char *key = readArgs("/KEY");
	if (key == 0 || strlen(key) == 0){
		MLOG( ERROR_LOGLVL ) " No decryption/encryption key given\n");

		errorCode |= EC_KEY_MISSING;

	}else if ( cipher!=NULL && stricmp(cipher, AES_STANDARD_NAME)==0) {
    
              if (key !=NULL && strlen(key)!=16){
		          MLOG( ERROR_LOGLVL ) "Incorrect decryption/encryption key size %d(Needed 16)\n",strlen(key));
			      errorCode |= EC_INCORRECT_SIZE;
	          }
          }else if ( cipher!=NULL && stricmp(cipher, DES_STANDARD_NAME)==0) {
    
              if (key !=NULL && strlen(key)!=8){

				  if (strlen(key) > 8){
					 MLOG( ERROR_LOGLVL ) "WARNING : Incorrect decryption/encryption key size %d will be truncated to 8 bytes\n",strlen(key));

				  }else{
					MLOG( ERROR_LOGLVL ) "Incorrect decryption/encryption key size %d(Needed 8)\n",strlen(key));
					errorCode |= EC_INCORRECT_SIZE;
				  }
	          }
          }
  char * help = readArgs( "/HELP" );
    if (help !=NULL && strlen(help)== 0 ) 
    {
        MLOG( INFO_LOGLVL ) "no help selected found\n");
                         
        //TODO : Added to list of errors
    }

   // MLOG(INFO_LOGLVL) "infile selected: <%s>", infile );
	fprintf(stderr, "Exiting checkArgs\n");
	return errorCode;

}


bool checkFileExists(const char *filename)
{   
    bool exists = true;
    struct stat fs;
    //  Check whether the read exists
    if (stat(filename, &fs)!= 0 ) {
        exists=false;

    }

    return exists;
}

bool  processFile(   const char *optmode,
                     const char *inputfile,
                     const char *outputfile,
                     const char *key,
                     const char *override,
					 const char *cipher,
                     const char *padlogfile)
{

	MLOG(INFO_LOGLVL) "Processfile in=<%s> out=<%s> mode=<%s> key=<%s> encryption=<%s>",inputfile, outputfile, optmode, key, cipher );

    if (stricmp(optmode,"encrypt")==0 ) {
        return encryptfile(inputfile, outputfile, key, override, cipher, padlogfile);
    }
    else if (stricmp(optmode,"decrypt")==0 ){
	     MLOG(INFO_LOGLVL) "decrypting in=<%s> out=<%s> key=<%s> encryption=<%s>",inputfile, outputfile, key, cipher );

         return decryptfile(inputfile, outputfile, key, override, cipher, padlogfile);
    }
	return false;
}

bool encryptfile( const char *inputfile,
             const char *outputfile,
             const char *key,
             const char *override,
			 const char *ciphername,
             const char *padlogfile)
{
   // TODO, : open a handle to the file  & access file details
   int defaultblocksize=DEFAULT_BLOCKSIZE;
   int rblocksize=0;
   int ifilehandle, ofilehandle;
   FILE * fpadlogfile=NULL;
   int ret=0;
   bool endoffile=false;
   char *rblock, *wblock;
   int byteread=0;
   bool BLOCKREADS=true;

   int padblock=16;

   if (stricmp (ciphername, AES_STANDARD_NAME)==0) {
       padblock=16;
   } else if (stricmp(ciphername, DES_STANDARD_NAME)==0) {
               padblock=8;

   }

 
   ifilehandle=_open(inputfile, O_RDWR|O_BINARY);
   if (ifilehandle == -1) // Inputfile open failed 
   {
		MLOG(ERROR) "InputFile %s could not be open errno %d", inputfile , errno);
		return false;
   }
   ofilehandle=_open(outputfile, O_CREAT|O_TRUNC|O_RDWR|O_BINARY,_S_IFMT |_S_IREAD |_S_IWRITE | _S_IEXEC );
   if (ofilehandle == -1) // Outputfile open failed 
   {
		MLOG(ERROR) "OutputFile %s could not be open errno %d", outputfile , errno);
		return false;
   }
   if (padlogfile != 0 &&strlen(padlogfile)>0) {
      fpadlogfile=fopen(padlogfile, "w");
  }

   rblocksize=defaultblocksize;
   rblock = new char[rblocksize+padblock];
   wblock = new char[rblocksize+padblock];
   
   loadfuncs();
   setkey((char *)key, padblock);
   bool padded=false;
   char *pad = new char[padblock];
   int rpos=0;
   struct stat buf;
   fstat(ifilehandle, &buf);
   // TODO :  Either read the complete file into memmory or carry out the operation block by block
  //  memcpy(GCrypt::instance()->aeskey, key, 16);
    while (!endoffile) {
        byteread=readblock(rblocksize, ifilehandle,rblock );
		
		if (byteread==0 || byteread == -1){
			endoffile=true;
			continue;
		}
		if(fpadlogfile){
               fprintf(fpadlogfile, "Block of Size %d read from file\n", byteread);
		}

		if (byteread< rblocksize && rpos+byteread == buf.st_size){
			int padLength = padblock - byteread % padblock;
			memset( pad, padLength, padblock );
			memcpy(rblock+byteread, pad, padLength);
			byteread+=padLength;
            if (fpadlogfile) {
                displayPadding(fpadlogfile, padblock,padLength, byteread, rblock);
            }


			} else if (rpos+byteread == buf.st_size){
				int padLength = padblock - byteread % padblock;
				memset( pad, padLength, padblock );
				memcpy(rblock+byteread, pad, padLength);
				byteread+=padLength;
                if (fpadlogfile) {
                            displayPadding(fpadlogfile, padblock,padLength, byteread, rblock);
                }


			}
		// AF : Assume that OS always delivers the maximum number of bytes on each read expect the last read.
		if (byteread <rblocksize) { 
		}

        ret=encrypt( (unsigned char*) wblock, byteread, 
                                     (unsigned char*) rblock, byteread, ciphername );
        if (ret==0) {
			if(fpadlogfile){
               fprintf(fpadlogfile, "Block of Size %d encrypted\n", byteread);
		}
            int written=writeblock(byteread, ofilehandle, wblock);
			MLOG(ERROR) "bytes written %d", written);

        }  else {
			MLOG(ERROR) "Encrpt error code %d", ret);
			ret=0;

			endoffile=true;
               //TODO: Operation failed, probably best to abort operation and report error
        }
		rpos+=byteread;
    }
	if (ofilehandle)
		close(ofilehandle);
	if (ifilehandle)
		close(ifilehandle);
    if(fpadlogfile!=NULL)
        fclose( fpadlogfile);

    if (rblock) {
        delete rblock;
    }
    if (wblock) {
        delete wblock;
    }
	if(pad){
		delete pad;
	}
   return (ret==0);
}

bool decryptfile( const char *inputfile,
             const char *outputfile,
             const char *key,
             const char *override,
			 const char *ciphername,
             const char *padlogfile)
{
   // TODO, : open a handle to the file  & access file details
   int defaultblocksize=DEFAULT_BLOCKSIZE;
   int rblocksize=0;
   int ret=0;
   char *rblock;
   char *wblock;
   rblocksize=defaultblocksize;
   int ifilehandle, ofilehandle;
   FILE *fpadlogfile = NULL;
   rblock = new char[rblocksize];
   wblock = new char[rblocksize];
   int byteread=0;
   int padblock=16;

   if (stricmp (ciphername, AES_STANDARD_NAME)==0) {
       padblock=16;
   } else if (stricmp(ciphername, DES_STANDARD_NAME)==0) {
               padblock=8;

   }
            
   ifilehandle=_open(inputfile, O_RDWR|O_BINARY);
   if (ifilehandle == -1) // Inputfile _open failed 
   {
		MLOG(ERROR_LOGLVL) "InputFile %s could not be open errno %d", inputfile , errno);
		return false;
   }
   ofilehandle=_open(outputfile, O_CREAT|O_TRUNC|O_RDWR|O_BINARY,_S_IREAD |_S_IWRITE );
   if (ofilehandle == -1) // Outputfile open failed 
   {
		MLOG(ERROR_LOGLVL) "OutputFile %s could not be open errno %d", outputfile , errno);
		return false;
   }
   MLOG(INFO_LOGLVL) "ifile %d ofile %d", ifilehandle, ofilehandle );
   if (padlogfile!= 0 && strlen(padlogfile)>0) {
       fpadlogfile=fopen(padlogfile, "w");
   }
   loadfuncs();
   bool unpadded=false;
   setkey((char *)key, padblock);
   struct stat buf;
   fstat (ifilehandle, &buf);
   bool endoffile=false;
   int rpos=0;
    while (!endoffile) {
        byteread=readblock(rblocksize, ifilehandle,rblock );
		if (byteread==0 || byteread == -1){
			endoffile=true;
			continue;
		}
		
		MLOG(INFO_LOGLVL) "read %d bytes",byteread );

        ret=decrypt( (unsigned char*) wblock, byteread, 
                                     (unsigned char*) rblock, byteread, ciphername );
		MLOG(INFO_LOGLVL) "decrypt ret %d",ret );

        if (ret==0) {
			MLOG( DEBUG_LOGLVL ) "::byteread  = %d, blocksize %d", byteread, rblocksize );
			if(fpadlogfile){
               fprintf(fpadlogfile, "Block of Size %d decrypted \n", byteread);
			}
			if (byteread< rblocksize && rpos+byteread == buf.st_size){
			    int padLength = wblock[ byteread - 1 ];
                MLOG( DEBUG_LOGLVL ) "::decrypt pad length = %d", padLength );
                if (fpadlogfile) {
                    displayPadding(fpadlogfile, padblock, padLength, byteread, wblock);
                }
                byteread-= padLength;
			} else if (rpos+byteread == buf.st_size){
					    int padLength = wblock[ byteread - 1 ];
						MLOG( DEBUG_LOGLVL ) "::decrypt pad length = %d", padLength );
                        if (fpadlogfile) {
                            displayPadding(fpadlogfile, padblock, padLength, byteread, wblock);
                        }
						byteread-= padLength;
			}
            if(fpadlogfile){
               fprintf(fpadlogfile, "Outputing Block of Size %d \n", byteread);
           }

            writeblock( byteread, ofilehandle,wblock);

        }  else {
			MLOG(ERROR_LOGLVL) "decrypt error code %d", ret);
               //TODO: Operation failed, probably best to abort operation and report error
        }
		rpos+=byteread;
    }

	if (ofilehandle)
		close(ofilehandle);
	if (ifilehandle)
		close(ifilehandle);
    if(fpadlogfile!=NULL)
        fclose( fpadlogfile);
    if (rblock) {
        delete rblock;
    }
    if (wblock) {
        delete wblock;
    }
   return (ret==0);
}

void displayPadding(FILE *fpadlogfile, int padblock, int padLength, int byteread, const char *dispblock){
    if (fpadlogfile !=NULL) {
                  fprintf(fpadlogfile, " Bytes Padded %d \n", padLength);
                  int startpad = ((byteread-padblock)>0)?byteread-padblock:0;
                  for ( int ii=startpad; ii < byteread ; ii++) {
                      unsigned char cc = ( unsigned char)*dispblock;
                     // fprintf(fpadlogfile, " <%02d> ", (int)cc);

                      fprintf(fpadlogfile, " <%02x> ", dispblock[ii]);
					 // wblock++;
                  }
                  fprintf(fpadlogfile,"\n");
    }

}

int readblock(int rblocksize, int ifilehandle,char *rblock )
{
	return read(ifilehandle, rblock, rblocksize);
}


int writeblock(int wblocksize, int ofilehandle,char *wblock )
{
	return write(ofilehandle, wblock, wblocksize);
}




